@@ -38,6 +38,10 @@
 /* This is the precision of coordinates in the database */
 #define EPSILON 0.000001
 
+/* This is the maximum distance for which we will attach an
+ * airport to a city, see also test_distance() */
+#define AIRPORT_MAX_DISTANCE 100.0
+
 /**
  * SECTION:gweatherlocation
  * @Title: GWeatherLocation
@@ -58,9 +62,6 @@
  * @GWEATHER_LOCATION_ADM1: A location representing a "first-level
  * administrative division"; ie, a state, province, or similar
  * division.
- * @GWEATHER_LOCATION_ADM2: A location representing a subdivision of a
- * %GWEATHER_LOCATION_ADM1 location, or a direct subdivision of
- * a country that is not represented in a #GWeatherLocationEntry.
  * @GWEATHER_LOCATION_CITY: A location representing a city
  * @GWEATHER_LOCATION_WEATHER_STATION: A location representing a
  * weather station.
@@ -163,6 +164,13 @@ add_nearest_weather_station (GWeatherLocation *location)
         if (siblings[i] == location)
             continue;
 
+        if (siblings[i]->level != GWEATHER_LOCATION_WEATHER_STATION)
+            continue;
+
+        /* Skip siblings without valid coordinates */
+        if (!siblings[i]->latlon_valid)
+            continue;
+
         distance = gweather_location_get_distance (location, siblings[i]);
         if (distance < min_distance) {
             closest = siblings[i];
@@ -170,11 +178,21 @@ add_nearest_weather_station (GWeatherLocation *location)
         }
     }
 
+    /* This should not happen */
     if (!closest) {
         g_critical ("Location '%s' has no valid airports attached", location->english_name);
         return;
     }
 
+    /* This could however */
+    if (min_distance > AIRPORT_MAX_DISTANCE) {
+        g_debug ("Not adding airport '%s' as it's too far from '%s' (%.1lf km)\n",
+                 gweather_location_get_name (closest),
+                 gweather_location_get_name (location),
+                 min_distance);
+        return;
+    }
+
     location->children = g_new0 (GWeatherLocation *, 2);
     location->children[0] = g_slice_new0 (GWeatherLocation);
     station = location->children[0];
@@ -184,7 +202,7 @@ add_nearest_weather_station (GWeatherLocation *location)
     station->local_sort_name = g_strdup (closest->local_sort_name);
     station->english_sort_name = g_strdup (closest->english_sort_name);
     station->parent = location;
-    station->level = closest->level;
+    station->level = GWEATHER_LOCATION_WEATHER_STATION;
     station->country_code = g_strdup (closest->country_code);
     station->tz_hint = g_strdup (closest->tz_hint);
     station->station_code = g_strdup (closest->station_code);
@@ -328,11 +346,6 @@ location_new_from_xml (GWeatherParser *parser, GWeatherLocationLevel level,
 	    if (!child)
 		goto error_out;
 	    g_ptr_array_add (children, child);
-	} else if (!strcmp (tagname, "province")) {
-	    child = location_new_from_xml (parser, GWEATHER_LOCATION_ADM2, loc);
-	    if (!child)
-		goto error_out;
-	    g_ptr_array_add (children, child);
 	} else if (!strcmp (tagname, "city")) {
 	    child = location_new_from_xml (parser, GWEATHER_LOCATION_CITY, loc);
 	    if (!child)
@@ -409,7 +422,15 @@ error_out:
     return NULL;
 }
 
-static GWeatherLocation *global_world;
+static GWeatherLocation *global_world = NULL;
+
+static void _gweather_location_unref_no_check (GWeatherLocation *loc);
+
+GWEATHER_EXTERN void
+_gweather_location_reset_world (void)
+{
+	g_clear_pointer (&global_world, _gweather_location_unref_no_check);
+}
 
 /**
  * gweather_location_get_world:
@@ -445,7 +466,8 @@ gweather_location_get_world (void)
 	    return NULL;
 
 	global_world = location_new_from_xml (parser, GWEATHER_LOCATION_WORLD, NULL);
-	add_nearest_weather_stations (global_world);
+	if (!g_getenv ("LIBGWEATHER_LOCATIONS_NO_NEAREST"))
+	    add_nearest_weather_stations (global_world);
 	_gweather_parser_free (parser);
     }
 
@@ -469,25 +491,14 @@ gweather_location_ref (GWeatherLocation *loc)
     return loc;
 }
 
-/**
- * gweather_location_unref:
- * @loc: a #GWeatherLocation
- *
- * Subtracts 1 from @loc's reference count, and frees it if the
- * reference count reaches 0.
- **/
-void
-gweather_location_unref (GWeatherLocation *loc)
+static void
+_gweather_location_unref_no_check (GWeatherLocation *loc)
 {
     int i;
 
-    g_return_if_fail (loc != NULL);
-
     if (--loc->ref_count)
 	return;
 
-    g_return_if_fail (loc->level != GWEATHER_LOCATION_WORLD);
-
     g_free (loc->english_name);
     g_free (loc->local_name);
     g_free (loc->msgctxt);
@@ -523,6 +534,22 @@ gweather_location_unref (GWeatherLocation *loc)
     g_slice_free (GWeatherLocation, loc);
 }
 
+/**
+ * gweather_location_unref:
+ * @loc: a #GWeatherLocation
+ *
+ * Subtracts 1 from @loc's reference count, and frees it if the
+ * reference count reaches 0.
+ **/
+void
+gweather_location_unref (GWeatherLocation *loc)
+{
+    g_return_if_fail (loc != NULL);
+    g_return_if_fail (loc->level != GWEATHER_LOCATION_WORLD);
+
+    _gweather_location_unref_no_check (loc);
+}
+
 GType
 gweather_location_get_type (void)
 {
@@ -588,6 +615,42 @@ gweather_location_get_level (GWeatherLocation *loc)
     return loc->level;
 }
 
+/**
+ * gweather_location_level_to_string:
+ * @level: a #GWeatherLocationLevel
+ *
+ * Returns the location level as a string, useful for debugging
+ * purposes.
+ *
+ * Return value: a string
+ **/
+const char *
+gweather_location_level_to_string (GWeatherLocationLevel level)
+{
+    switch (level) {
+    case GWEATHER_LOCATION_WORLD:
+        return "world";
+    case GWEATHER_LOCATION_REGION:
+        return "region";
+    case GWEATHER_LOCATION_COUNTRY:
+        return "country";
+    case GWEATHER_LOCATION_ADM1:
+        return "adm1";
+    case GWEATHER_LOCATION_CITY:
+        return "city";
+    case GWEATHER_LOCATION_WEATHER_STATION:
+        return "station";
+    case GWEATHER_LOCATION_DETACHED:
+        return "detached";
+    case GWEATHER_LOCATION_NAMED_TIMEZONE:
+        return "named-timezone";
+    default:
+        g_assert_not_reached();
+    }
+
+    return NULL;
+}
+
 /**
  * gweather_location_get_parent:
  * @loc: a #GWeatherLocation
@@ -708,7 +771,7 @@ find_nearest_city (GWeatherLocation *location,
  * Finds the nearest city to the passed latitude and
  * longitude, among the descendants of @loc.
  *
- * @loc must be at most a %GWEATHER_LOCATION_ADM2 location.
+ * @loc must be at most a %GWEATHER_LOCATION_ADM1 location.
  * This restriction may be lifted in a future version.
  *
  * Note that this function does not check if (@lat, @lon) fall inside
@@ -763,7 +826,7 @@ gweather_location_find_nearest_city (GWeatherLocation *loc,
  * Supports the use of own filter function to filter out locations.
  * Geocoding should be done on the application side if needed.
  *
- * @loc must be at most a %GWEATHER_LOCATION_ADM2 location.
+ * @loc must be at most a %GWEATHER_LOCATION_ADM1 location.
  * This restriction may be lifted in a future version.
  *
  * Returns: (transfer full): the city closest to (@lat, @lon), in the
@@ -859,7 +922,7 @@ _got_place (GObject      *source_object,
  * Initializes geocode reversing to find place for (@lat, @lon) coordinates. Calls the callback
  * function passed by user when the result is ready.
  *
- * @loc must be at most a %GWEATHER_LOCATION_ADM2 location.
+ * @loc must be at most a %GWEATHER_LOCATION_ADM1 location.
  * This restriction may be lifted in a future version.
  *
  * Since: 3.12
@@ -971,11 +1034,11 @@ gweather_location_get_coords (GWeatherLocation *loc,
 double
 gweather_location_get_distance (GWeatherLocation *loc, GWeatherLocation *loc2)
 {
-    g_return_val_if_fail (loc != NULL, 0);
-    g_return_val_if_fail (loc2 != NULL, 0);
+    g_return_val_if_fail (loc != NULL, G_MAXDOUBLE);
+    g_return_val_if_fail (loc2 != NULL, G_MAXDOUBLE);
 
-    //g_return_val_if_fail (loc->latlon_valid, 0.0);
-    //g_return_val_if_fail (loc2->latlon_valid, 0.0);
+    g_return_val_if_fail (loc->latlon_valid, G_MAXDOUBLE);
+    g_return_val_if_fail (loc2->latlon_valid, G_MAXDOUBLE);
 
     return location_distance (loc->latitude, loc->longitude,
 			      loc2->latitude, loc2->longitude);
@@ -1330,8 +1393,7 @@ gweather_location_equal (GWeatherLocation *one,
     if (level == GWEATHER_LOCATION_COUNTRY)
 	return g_strcmp0 (one->country_code, two->country_code);
 
-    if (level == GWEATHER_LOCATION_ADM1 ||
-	level == GWEATHER_LOCATION_ADM2) {
+    if (level == GWEATHER_LOCATION_ADM1) {
 	if (g_strcmp0 (one->english_sort_name, two->english_sort_name) != 0)
 	    return FALSE;
 
@@ -1367,8 +1429,9 @@ gweather_location_format_two_serialize (GWeatherLocation *location)
 
     /* Normalize location to be a weather station or detached */
     if (location->level == GWEATHER_LOCATION_CITY) {
-	location = location->children[0];
-	is_city = TRUE;
+        if (location->children != NULL)
+            location = location->children[0];
+        is_city = TRUE;
     } else {
 	is_city = FALSE;
     }
@@ -1382,7 +1445,7 @@ gweather_location_format_two_serialize (GWeatherLocation *location)
 	g_variant_builder_add (&parent_latlon_builder, "(dd)", location->parent->latitude, location->parent->longitude);
 
     return g_variant_new ("(ssba(dd)a(dd))",
-			  name, location->station_code, is_city,
+			  name, location->station_code ? location->station_code : "", is_city,
 			  &latlon_builder, &parent_latlon_builder);
 }
 
