@@ -406,8 +406,7 @@ build_yrno_url_geonames (GWeatherLocation *glocation,
     while (glocation) {
 	if (glocation->level == GWEATHER_LOCATION_CITY)
 	    city_name = glocation->english_name;
-	if (glocation->level == GWEATHER_LOCATION_ADM1 ||
-	    glocation->level == GWEATHER_LOCATION_ADM2)
+	if (glocation->level == GWEATHER_LOCATION_ADM1)
 	    adm_division = glocation->english_name;
 	if (glocation->level == GWEATHER_LOCATION_COUNTRY)
 	    country = glocation->english_name;
